var __defProp = Object.defineProperty;
var __require = /* @__PURE__ */ ((x) => typeof require !== "undefined" ? require : typeof Proxy !== "undefined" ? new Proxy(x, {
  get: (a, b) => (typeof require !== "undefined" ? require : a)[b]
}) : x)(function(x) {
  if (typeof require !== "undefined")
    return require.apply(this, arguments);
  throw Error('Dynamic require of "' + x + '" is not supported');
});
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// packages/native-federation-typescript/src/lib/typeScriptCompiler.ts
import ansiColors from "ansi-colors";
import { dirname, join, normalize, relative, resolve } from "path";
import typescript from "typescript";
import { ThirdPartyExtractor } from "@module-federation/third-party-dts-extractor";
var STARTS_WITH_SLASH = /^\//;
var DEFINITION_FILE_EXTENSION = ".d.ts";
var reportCompileDiagnostic = (diagnostic) => {
  const { line } = diagnostic.file.getLineAndCharacterOfPosition(
    diagnostic.start
  );
  console.error(
    ansiColors.red(
      `TS Error ${diagnostic.code}':' ${typescript.flattenDiagnosticMessageText(
        diagnostic.messageText,
        typescript.sys.newLine
      )}`
    )
  );
  console.error(
    ansiColors.red(
      `         at ${diagnostic.file.fileName}:${line + 1} typescript.sys.newLine`
    )
  );
};
var retrieveMfTypesPath = (tsConfig, remoteOptions) => normalize(tsConfig.outDir.replace(remoteOptions.compiledTypesFolder, ""));
var retrieveOriginalOutDir = (tsConfig, remoteOptions) => normalize(
  tsConfig.outDir.replace(remoteOptions.compiledTypesFolder, "").replace(remoteOptions.typesFolder, "")
);
var retrieveMfAPITypesPath = (tsConfig, remoteOptions) => join(
  retrieveOriginalOutDir(tsConfig, remoteOptions),
  `${remoteOptions.typesFolder}.d.ts`
);
var createHost = (mapComponentsToExpose, tsConfig, remoteOptions, cb) => {
  const host = typescript.createCompilerHost(tsConfig);
  const originalWriteFile = host.writeFile;
  const mapExposeToEntry = Object.fromEntries(
    Object.entries(mapComponentsToExpose).map((entry) => entry.reverse())
  );
  const mfTypePath = retrieveMfTypesPath(tsConfig, remoteOptions);
  host.writeFile = (filepath, text, writeOrderByteMark, onError, sourceFiles, data) => {
    originalWriteFile(
      filepath,
      text,
      writeOrderByteMark,
      onError,
      sourceFiles,
      data
    );
    for (const sourceFile of sourceFiles || []) {
      const sourceEntry = mapExposeToEntry[sourceFile.fileName];
      if (sourceEntry) {
        const mfeTypeEntry = join(
          mfTypePath,
          `${sourceEntry}${DEFINITION_FILE_EXTENSION}`
        );
        const mfeTypeEntryDirectory = dirname(mfeTypeEntry);
        const relativePathToOutput = relative(mfeTypeEntryDirectory, filepath).replace(DEFINITION_FILE_EXTENSION, "").replace(STARTS_WITH_SLASH, "");
        originalWriteFile(
          mfeTypeEntry,
          `export * from './${relativePathToOutput}';
export { default } from './${relativePathToOutput}';`,
          writeOrderByteMark
        );
      }
    }
    cb(text);
  };
  return host;
};
var createVueTscProgram = (programOptions) => {
  const vueTypescript = __require("vue-tsc");
  return vueTypescript.createProgram(programOptions);
};
var createProgram = (remoteOptions, programOptions) => {
  switch (remoteOptions.compilerInstance) {
    case "vue-tsc":
      return createVueTscProgram(programOptions);
    case "tsc":
    default:
      return typescript.createProgram(programOptions);
  }
};
var compileTs = (mapComponentsToExpose, tsConfig, remoteOptions) => {
  const mfTypePath = retrieveMfTypesPath(tsConfig, remoteOptions);
  const thirdPartyExtractor = new ThirdPartyExtractor(
    resolve(mfTypePath, "node_modules"),
    remoteOptions.context
  );
  const cb = remoteOptions.extractThirdParty ? thirdPartyExtractor.collectPkgs.bind(thirdPartyExtractor) : () => void 0;
  const tsHost = createHost(mapComponentsToExpose, tsConfig, remoteOptions, cb);
  const filesToCompile = [
    ...Object.values(mapComponentsToExpose),
    ...remoteOptions.additionalFilesToCompile
  ];
  const programOptions = {
    rootNames: filesToCompile,
    host: tsHost,
    options: tsConfig
  };
  const tsProgram = createProgram(remoteOptions, programOptions);
  const { diagnostics = [] } = tsProgram.emit();
  diagnostics.forEach(reportCompileDiagnostic);
  if (remoteOptions.extractThirdParty) {
    thirdPartyExtractor.copyDts();
  }
};

// packages/native-federation-typescript/src/lib/archiveHandler.ts
import AdmZip from "adm-zip";
import ansiColors4 from "ansi-colors";
import axios2 from "axios";
import { resolve as resolve3, join as join3 } from "path";

// packages/native-federation-typescript/src/lib/utils.ts
import path2 from "path";

// packages/native-federation-typescript/src/configurations/remotePlugin.ts
import { existsSync } from "fs";
import { dirname as dirname2, join as join2, resolve as resolve2, extname } from "path";
import { utils } from "@module-federation/managers";
import typescript2 from "typescript";
var defaultOptions = {
  tsConfigPath: "./tsconfig.json",
  typesFolder: "@mf-types",
  compiledTypesFolder: "compiled-types",
  hostRemoteTypesFolder: "@mf-types",
  deleteTypesFolder: true,
  additionalFilesToCompile: [],
  compilerInstance: "tsc",
  compileInChildProcess: false,
  implementation: "",
  generateAPITypes: false,
  context: process.cwd(),
  abortOnError: true,
  extractRemoteTypes: false,
  extractThirdParty: false
};
var readTsConfig = ({
  tsConfigPath,
  typesFolder,
  compiledTypesFolder,
  context
}) => {
  const resolvedTsConfigPath = resolve2(context, tsConfigPath);
  const readResult = typescript2.readConfigFile(
    resolvedTsConfigPath,
    typescript2.sys.readFile
  );
  if (readResult.error) {
    throw new Error(readResult.error.messageText.toString());
  }
  const configContent = typescript2.parseJsonConfigFileContent(
    readResult.config,
    typescript2.sys,
    dirname2(resolvedTsConfigPath)
  );
  const outDir = resolve2(
    context,
    configContent.options.outDir || "dist",
    typesFolder,
    compiledTypesFolder
  );
  return {
    ...configContent.options,
    emitDeclarationOnly: true,
    noEmit: false,
    declaration: true,
    outDir
  };
};
var TS_EXTENSIONS = ["ts", "tsx", "vue", "svelte"];
var resolveWithExtension = (exposedPath, context) => {
  if (extname(exposedPath)) {
    return resolve2(context, exposedPath);
  }
  for (const extension of TS_EXTENSIONS) {
    const exposedPathWithExtension = resolve2(
      context,
      `${exposedPath}.${extension}`
    );
    if (existsSync(exposedPathWithExtension)) {
      return exposedPathWithExtension;
    }
  }
  return void 0;
};
var resolveExposes = (remoteOptions) => {
  const parsedOptions = utils.parseOptions(
    remoteOptions.moduleFederationConfig.exposes || {},
    (item, key) => ({
      exposePath: Array.isArray(item) ? item[0] : item,
      key
    }),
    (item, key) => ({
      exposePath: Array.isArray(item.import) ? item.import[0] : item.import[0],
      key
    })
  );
  return parsedOptions.reduce(
    (accumulator, item) => {
      const { exposePath, key } = item[1];
      accumulator[key] = resolveWithExtension(exposePath, remoteOptions.context) || resolveWithExtension(
        join2(exposePath, "index"),
        remoteOptions.context
      ) || exposePath;
      return accumulator;
    },
    {}
  );
};
var retrieveRemoteConfig = (options) => {
  validateOptions(options);
  const remoteOptions = {
    ...defaultOptions,
    ...options
  };
  const mapComponentsToExpose = resolveExposes(remoteOptions);
  const tsConfig = readTsConfig(remoteOptions);
  return {
    tsConfig,
    mapComponentsToExpose,
    remoteOptions
  };
};

// packages/native-federation-typescript/src/lib/DTSManager.ts
import ansiColors2 from "ansi-colors";
import path from "path";
import { rm } from "fs/promises";
import fs from "fs";
import { MANIFEST_EXT as MANIFEST_EXT2 } from "@module-federation/sdk";
import cloneDeepWith from "lodash.clonedeepwith";

// packages/native-federation-typescript/src/configurations/hostPlugin.ts
import { MANIFEST_EXT, parseEntry } from "@module-federation/sdk";
import { utils as utils2 } from "@module-federation/managers";
var defaultOptions2 = {
  typesFolder: "@mf-types",
  remoteTypesFolder: "@mf-types",
  deleteTypesFolder: true,
  maxRetries: 3,
  implementation: "",
  context: process.cwd(),
  abortOnError: true,
  consumeAPITypes: false
};
var buildZipUrl = (hostOptions, url) => {
  const remoteUrl = new URL(url);
  if (remoteUrl.href.includes(MANIFEST_EXT)) {
    return void 0;
  }
  const pathnameWithoutEntry = remoteUrl.pathname.split("/").slice(0, -1).join("/");
  remoteUrl.pathname = `${pathnameWithoutEntry}/${hostOptions.remoteTypesFolder}.zip`;
  return remoteUrl.href;
};
var buildApiTypeUrl = (zipUrl) => {
  if (!zipUrl) {
    return void 0;
  }
  return zipUrl.replace(".zip", ".d.ts");
};
var retrieveRemoteInfo = (options) => {
  const { hostOptions, remoteAlias, remote } = options;
  const parsedInfo = parseEntry(remote, void 0, "@");
  const url = "entry" in parsedInfo ? parsedInfo.entry : parsedInfo.name === remote ? remote : "";
  const zipUrl = url ? buildZipUrl(hostOptions, url) : "";
  return {
    name: parsedInfo.name || remoteAlias,
    url,
    zipUrl,
    apiTypeUrl: buildApiTypeUrl(zipUrl),
    alias: remoteAlias
  };
};
var resolveRemotes = (hostOptions) => {
  const parsedOptions = utils2.parseOptions(
    hostOptions.moduleFederationConfig.remotes || {},
    (item, key) => ({
      remote: Array.isArray(item) ? item[0] : item,
      key
    }),
    (item, key) => ({
      remote: Array.isArray(item.external) ? item.external[0] : item.external,
      key
    })
  );
  return parsedOptions.reduce(
    (accumulator, item) => {
      const { key, remote } = item[1];
      accumulator[key] = retrieveRemoteInfo({
        hostOptions,
        remoteAlias: key,
        remote
      });
      return accumulator;
    },
    {}
  );
};
var retrieveHostConfig = (options) => {
  validateOptions(options);
  const hostOptions = { ...defaultOptions2, ...options };
  const mapRemotesToDownload = resolveRemotes(hostOptions);
  return {
    hostOptions,
    mapRemotesToDownload
  };
};

// packages/native-federation-typescript/src/constant.ts
var REMOTE_ALIAS_IDENTIFIER = "REMOTE_ALIAS_IDENTIFIER";
var REMOTE_API_TYPES_FILE_NAME = "apis.d.ts";
var HOST_API_TYPES_FILE_NAME = "index.d.ts";
var UpdateMode = /* @__PURE__ */ ((UpdateMode2) => {
  UpdateMode2["POSITIVE"] = "POSITIVE";
  UpdateMode2["PASSIVE"] = "PASSIVE";
  return UpdateMode2;
})(UpdateMode || {});

// packages/native-federation-typescript/src/lib/DTSManager.ts
import axios from "axios";
var DTSManager = class {
  constructor(options) {
    this.options = cloneDeepWith(options, (_value, key) => {
      if (key === "manifest") {
        return false;
      }
    });
    this.runtimePkgs = [
      "@module-federation/runtime",
      "@module-federation/runtime-tools"
    ];
    this.loadedRemoteAPIAlias = [];
    this.remoteAliasMap = {};
    this.extraOptions = (options == null ? void 0 : options.extraOptions) || {};
  }
  generateAPITypes(mapComponentsToExpose) {
    const exposePaths = /* @__PURE__ */ new Set();
    const packageType = Object.keys(mapComponentsToExpose).reduce(
      (sum, exposeKey) => {
        const exposePath = path.join(REMOTE_ALIAS_IDENTIFIER, exposeKey);
        exposePaths.add(`'${exposePath}'`);
        const curType = `T extends '${exposePath}' ? typeof import('${exposePath}') :`;
        sum = curType + sum;
        return sum;
      },
      "any;"
    );
    const exposePathKeys = [...exposePaths].join(" | ");
    return `
    export type RemoteKeys = ${exposePathKeys};
    type PackageType<T> = ${packageType}`;
  }
  async extractRemoteTypes(options) {
    const { remoteOptions, tsConfig } = options;
    if (!remoteOptions.extractRemoteTypes) {
      return;
    }
    let hasRemotes = false;
    const remotes = remoteOptions.moduleFederationConfig.remotes;
    if (remotes) {
      if (Array.isArray(remotes)) {
        hasRemotes = Boolean(remotes.length);
      } else if (typeof remotes === "object") {
        hasRemotes = Boolean(Object.keys(remotes).length);
      }
    }
    const mfTypesPath = retrieveMfTypesPath(tsConfig, remoteOptions);
    if (hasRemotes) {
      const tempHostOptions = {
        moduleFederationConfig: remoteOptions.moduleFederationConfig,
        typesFolder: path.join(mfTypesPath, "node_modules"),
        remoteTypesFolder: (remoteOptions == null ? void 0 : remoteOptions.hostRemoteTypesFolder) || remoteOptions.typesFolder,
        deleteTypesFolder: true,
        context: remoteOptions.context,
        implementation: remoteOptions.implementation,
        abortOnError: false
      };
      await this.consumeArchiveTypes(tempHostOptions);
    }
  }
  async generateTypes() {
    var _a;
    try {
      const { options } = this;
      if (!options.remote) {
        throw new Error(
          "options.remote is required if you want to generateTypes"
        );
      }
      const { remoteOptions, tsConfig, mapComponentsToExpose } = retrieveRemoteConfig(options.remote);
      if (!Object.keys(mapComponentsToExpose).length) {
        return;
      }
      this.extractRemoteTypes({
        remoteOptions,
        tsConfig,
        mapComponentsToExpose
      });
      compileTs(mapComponentsToExpose, tsConfig, remoteOptions);
      await createTypesArchive(tsConfig, remoteOptions);
      let apiTypesPath = "";
      if (remoteOptions.generateAPITypes) {
        const apiTypes = this.generateAPITypes(mapComponentsToExpose);
        apiTypesPath = retrieveMfAPITypesPath(tsConfig, remoteOptions);
        fs.writeFileSync(apiTypesPath, apiTypes);
      }
      if (remoteOptions.deleteTypesFolder) {
        await rm(retrieveMfTypesPath(tsConfig, remoteOptions), {
          recursive: true,
          force: true
        });
      }
      console.log(ansiColors2.green("Federated types created correctly"));
    } catch (error) {
      if (((_a = this.options.remote) == null ? void 0 : _a.abortOnError) === false) {
        console.error(
          ansiColors2.red(`Unable to compile federated types, ${error}`)
        );
      } else {
        throw error;
      }
    }
  }
  async requestRemoteManifest(remoteInfo) {
    try {
      if (!remoteInfo.url.includes(MANIFEST_EXT2)) {
        return remoteInfo;
      }
      const url = replaceLocalhost(remoteInfo.url);
      const res = await axios({
        method: "get",
        url
      });
      const manifestJson = res.data;
      if (!manifestJson.metaData.types.zip) {
        throw new Error(`Can not get ${remoteInfo.name}'s types archive url!`);
      }
      const addProtocol = (u) => {
        if (u.startsWith("//")) {
          return `https:${u}`;
        }
        return u;
      };
      const publicPath = "publicPath" in manifestJson.metaData ? manifestJson.metaData.publicPath : new Function(manifestJson.metaData.getPublicPath)();
      remoteInfo.zipUrl = new URL(
        path.join(addProtocol(publicPath), manifestJson.metaData.types.zip)
      ).href;
      if (!manifestJson.metaData.types.api) {
        console.warn(`Can not get ${remoteInfo.name}'s api types url!`);
        remoteInfo.apiTypeUrl = "";
        return remoteInfo;
      }
      remoteInfo.apiTypeUrl = new URL(
        path.join(addProtocol(publicPath), manifestJson.metaData.types.api)
      ).href;
      return remoteInfo;
    } catch (_err) {
      console.error(_err);
      return remoteInfo;
    }
  }
  async consumeTargetRemotes(hostOptions, remoteInfo) {
    if (!remoteInfo.zipUrl) {
      throw new Error(`Can not get ${remoteInfo.name}'s types archive url!`);
    }
    const typesDownloader = downloadTypesArchive(hostOptions);
    return typesDownloader([remoteInfo.alias, remoteInfo.zipUrl]);
  }
  async downloadAPITypes(remoteInfo, destinationPath) {
    const { apiTypeUrl } = remoteInfo;
    if (!apiTypeUrl) {
      return;
    }
    try {
      const url = replaceLocalhost(apiTypeUrl);
      const res = await axios.get(url);
      let apiTypeFile = res.data;
      apiTypeFile = apiTypeFile.replaceAll(
        REMOTE_ALIAS_IDENTIFIER,
        remoteInfo.alias
      );
      const filePath = path.join(destinationPath, REMOTE_API_TYPES_FILE_NAME);
      fs.writeFileSync(filePath, apiTypeFile);
      this.loadedRemoteAPIAlias.push(remoteInfo.alias);
    } catch (err) {
      console.error(
        ansiColors2.red(
          `Unable to download "${remoteInfo.name}" api types, ${err}`
        )
      );
    }
  }
  consumeAPITypes(hostOptions) {
    if (!this.loadedRemoteAPIAlias.length) {
      return;
    }
    const packageTypes = [];
    const remoteKeys = [];
    const importTypeStr = this.loadedRemoteAPIAlias.map((alias, index) => {
      const remoteKey = `RemoteKeys_${index}`;
      const packageType = `PackageType_${index}`;
      packageTypes.push(`T extends ${remoteKey} ? ${packageType}<T>`);
      remoteKeys.push(remoteKey);
      return `import type { PackageType as ${packageType},RemoteKeys as ${remoteKey} } from './${alias}/apis.d.ts';`;
    }).join("\n");
    const remoteKeysStr = `type RemoteKeys = ${remoteKeys.join(" | ")};`;
    const packageTypesStr = `type PackageType<T, Y=any> = ${[
      ...packageTypes,
      "Y"
    ].join(" :\n")} ;`;
    const pkgsDeclareStr = this.runtimePkgs.map((pkg) => {
      return `declare module "${pkg}" {
      ${remoteKeysStr}
      ${packageTypesStr}
      export function loadRemote<T extends RemoteKeys,Y>(packageName: T): Promise<PackageType<T, Y>>;
      export function loadRemote<T extends string,Y>(packageName: T): Promise<PackageType<T, Y>>;
    }`;
    }).join("\n");
    const fileStr = `${importTypeStr}
    ${pkgsDeclareStr}
    `;
    fs.writeFileSync(
      path.join(
        hostOptions.context,
        hostOptions.typesFolder,
        HOST_API_TYPES_FILE_NAME
      ),
      fileStr
    );
  }
  async consumeArchiveTypes(options) {
    const { hostOptions, mapRemotesToDownload } = retrieveHostConfig(options);
    if (hostOptions.deleteTypesFolder) {
      await rm(hostOptions.typesFolder, {
        recursive: true,
        force: true
      }).catch(
        (error) => console.error(
          ansiColors2.red(`Unable to remove types folder, ${error}`)
        )
      );
    }
    const downloadPromises = Object.entries(mapRemotesToDownload).map(
      async (item) => {
        const remoteInfo = item[1];
        if (!this.remoteAliasMap[remoteInfo.alias]) {
          const requiredRemoteInfo = await this.requestRemoteManifest(remoteInfo);
          this.remoteAliasMap[remoteInfo.alias] = requiredRemoteInfo;
        }
        return this.consumeTargetRemotes(
          hostOptions,
          this.remoteAliasMap[remoteInfo.alias]
        );
      }
    );
    const downloadPromisesResult = await Promise.allSettled(downloadPromises);
    return {
      hostOptions,
      downloadPromisesResult
    };
  }
  async consumeTypes() {
    var _a;
    try {
      const { options } = this;
      if (!options.host) {
        throw new Error("options.host is required if you want to consumeTypes");
      }
      const { mapRemotesToDownload } = retrieveHostConfig(options.host);
      if (!Object.keys(mapRemotesToDownload).length) {
        return;
      }
      const { downloadPromisesResult, hostOptions } = await this.consumeArchiveTypes(options.host);
      if (hostOptions.consumeAPITypes) {
        await Promise.all(
          downloadPromisesResult.map(async (item) => {
            if (item.status === "rejected" || !item.value) {
              return;
            }
            const [alias, destinationPath] = item.value;
            const remoteInfo = this.remoteAliasMap[alias];
            if (!remoteInfo) {
              return;
            }
            await this.downloadAPITypes(remoteInfo, destinationPath);
          })
        );
        this.consumeAPITypes(hostOptions);
      }
      console.log(ansiColors2.green("Federated types extraction completed"));
    } catch (err) {
      if (((_a = this.options.host) == null ? void 0 : _a.abortOnError) === false) {
        console.error(
          ansiColors2.red(`Unable to consume federated types, ${err}`)
        );
      } else {
        throw err;
      }
    }
  }
  async updateTypes(options) {
    var _a, _b, _c;
    const { remoteName, updateMode } = options;
    const hostName = (_c = (_b = (_a = this.options) == null ? void 0 : _a.host) == null ? void 0 : _b.moduleFederationConfig) == null ? void 0 : _c.name;
    if (updateMode === "POSITIVE" /* POSITIVE */ && remoteName === hostName) {
      if (!this.options.remote) {
        return;
      }
      this.generateTypes();
    } else {
      const { remoteAliasMap } = this;
      if (!this.options.host) {
        return;
      }
      const { hostOptions, mapRemotesToDownload } = retrieveHostConfig(
        this.options.host
      );
      const loadedRemoteInfo = Object.values(remoteAliasMap).find(
        (i) => i.name === remoteName
      );
      if (!loadedRemoteInfo) {
        const remoteInfo = Object.values(mapRemotesToDownload).find((item) => {
          return item.name === remoteName;
        });
        if (remoteInfo) {
          if (!this.remoteAliasMap[remoteInfo.alias]) {
            const requiredRemoteInfo = await this.requestRemoteManifest(remoteInfo);
            this.remoteAliasMap[remoteInfo.alias] = requiredRemoteInfo;
          }
          await this.consumeTargetRemotes(
            hostOptions,
            this.remoteAliasMap[remoteInfo.alias]
          );
        }
      } else {
        await this.consumeTargetRemotes(hostOptions, loadedRemoteInfo);
      }
    }
  }
};

// packages/native-federation-typescript/src/lib/utils.ts
import ansiColors3 from "ansi-colors";
function getDTSManagerConstructor(implementation) {
  if (implementation) {
    const NewConstructor = __require(implementation);
    return NewConstructor.default ? NewConstructor.default : NewConstructor;
  }
  return DTSManager;
}
var validateOptions = (options) => {
  if (!options.moduleFederationConfig) {
    throw new Error("moduleFederationConfig is required");
  }
};
function retrieveTypesAssetsInfo(options) {
  let apiTypesPath = "";
  let zipTypesPath = "";
  try {
    const { tsConfig, remoteOptions, mapComponentsToExpose } = retrieveRemoteConfig(options);
    if (!Object.keys(mapComponentsToExpose).length) {
      return {
        apiTypesPath,
        zipTypesPath,
        zipName: "",
        apiFileName: ""
      };
    }
    const mfTypesPath = retrieveMfTypesPath(tsConfig, remoteOptions);
    zipTypesPath = retrieveTypesZipPath(mfTypesPath, remoteOptions);
    if (remoteOptions.generateAPITypes) {
      apiTypesPath = retrieveMfAPITypesPath(tsConfig, remoteOptions);
    }
    return {
      apiTypesPath,
      zipTypesPath,
      zipName: path2.basename(zipTypesPath),
      apiFileName: path2.basename(apiTypesPath)
    };
  } catch (err) {
    console.error(ansiColors3.red(`Unable to compile federated types, ${err}`));
    return {
      apiTypesPath: "",
      zipTypesPath: "",
      zipName: "",
      apiFileName: ""
    };
  }
}
function replaceLocalhost(url) {
  return url.replace("localhost", "127.0.0.1");
}
function isDebugMode() {
  return Boolean(process.env.FEDERATION_DEBUG);
}

// packages/native-federation-typescript/src/lib/archiveHandler.ts
var retrieveTypesZipPath = (mfTypesPath, remoteOptions) => join3(
  mfTypesPath.replace(remoteOptions.typesFolder, ""),
  `${remoteOptions.typesFolder}.zip`
);
var createTypesArchive = async (tsConfig, remoteOptions) => {
  const mfTypesPath = retrieveMfTypesPath(tsConfig, remoteOptions);
  const zip = new AdmZip();
  zip.addLocalFolder(mfTypesPath);
  return zip.writeZipPromise(retrieveTypesZipPath(mfTypesPath, remoteOptions));
};
var downloadErrorLogger = (destinationFolder, fileToDownload) => (reason) => {
  throw {
    ...reason,
    message: `Network error: Unable to download federated mocks for '${destinationFolder}' from '${fileToDownload}' because '${reason.message}'`
  };
};
var retrieveTypesArchiveDestinationPath = (hostOptions, destinationFolder) => {
  return resolve3(
    hostOptions.context,
    hostOptions.typesFolder,
    destinationFolder
  );
};
var downloadTypesArchive = (hostOptions) => {
  let retries = 0;
  return async ([destinationFolder, fileToDownload]) => {
    const destinationPath = retrieveTypesArchiveDestinationPath(
      hostOptions,
      destinationFolder
    );
    while (retries++ < hostOptions.maxRetries) {
      try {
        const url = replaceLocalhost(fileToDownload);
        const response = await axios2.get(url, { responseType: "arraybuffer" }).catch(downloadErrorLogger(destinationFolder, url));
        const zip = new AdmZip(Buffer.from(response.data));
        zip.extractAllTo(destinationPath, true);
        return [destinationFolder, destinationPath];
      } catch (error) {
        if (isDebugMode()) {
          console.error(
            ansiColors4.red(
              `Error during types archive download: ${(error == null ? void 0 : error.message) || "unknown error"}`
            )
          );
        }
        if (retries >= hostOptions.maxRetries) {
          if (hostOptions.abortOnError !== false) {
            throw error;
          }
          return void 0;
        }
      }
    }
  };
};

// packages/native-federation-typescript/src/lib/generateTypes.ts
async function generateTypes(options) {
  var _a;
  const DTSManagerConstructor = getDTSManagerConstructor(
    (_a = options.remote) == null ? void 0 : _a.implementation
  );
  const dtsManager = new DTSManagerConstructor(options);
  return dtsManager.generateTypes();
}

// packages/native-federation-typescript/src/rpc/types.ts
var RpcGMCallTypes = /* @__PURE__ */ ((RpcGMCallTypes2) => {
  RpcGMCallTypes2["CALL"] = "mf_call";
  RpcGMCallTypes2["RESOLVE"] = "mf_resolve";
  RpcGMCallTypes2["REJECT"] = "mf_reject";
  RpcGMCallTypes2["EXIT"] = "mf_exit";
  return RpcGMCallTypes2;
})(RpcGMCallTypes || {});

// packages/native-federation-typescript/src/rpc/expose-rpc.ts
import process2 from "process";
function exposeRpc(fn) {
  const sendMessage = (message) => new Promise((resolve4, reject) => {
    if (!process2.send) {
      reject(new Error(`Process ${process2.pid} doesn't have IPC channels`));
    } else if (!process2.connected) {
      reject(
        new Error(`Process ${process2.pid} doesn't have open IPC channels`)
      );
    } else {
      process2.send(message, void 0, void 0, (error) => {
        if (error) {
          reject(error);
        } else {
          resolve4(void 0);
        }
      });
    }
  });
  const handleMessage = async (message) => {
    if (message.type === "mf_call" /* CALL */) {
      if (!process2.send) {
        return;
      }
      let value, error;
      try {
        value = await fn(...message.args);
      } catch (fnError) {
        error = fnError;
      }
      try {
        if (error) {
          await sendMessage({
            type: "mf_reject" /* REJECT */,
            id: message.id,
            error
          });
        } else {
          await sendMessage({
            type: "mf_resolve" /* RESOLVE */,
            id: message.id,
            value
          });
        }
      } catch (sendError) {
        if (error) {
          if (error instanceof Error) {
            console.error(error);
          }
        }
        console.error(sendError);
      }
    }
  };
  process2.on("message", handleMessage);
}

export {
  __export,
  retrieveMfTypesPath,
  retrieveOriginalOutDir,
  retrieveTypesZipPath,
  retrieveHostConfig,
  REMOTE_ALIAS_IDENTIFIER,
  REMOTE_API_TYPES_FILE_NAME,
  HOST_API_TYPES_FILE_NAME,
  UpdateMode,
  DTSManager,
  getDTSManagerConstructor,
  validateOptions,
  retrieveTypesAssetsInfo,
  retrieveRemoteConfig,
  generateTypes,
  RpcGMCallTypes,
  exposeRpc
};
