"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { newObj[key] = obj[key]; } } } newObj.default = obj; return newObj; } } function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }




var _chunk2NC7LVYZjs = require('./chunk-2NC7LVYZ.js');

// packages/native-federation-typescript/src/lib/DtsWorker.ts
var _path = require('path'); var _path2 = _interopRequireDefault(_path);
var _lodashclonedeepwith = require('lodash.clonedeepwith'); var _lodashclonedeepwith2 = _interopRequireDefault(_lodashclonedeepwith);

// packages/native-federation-typescript/src/rpc/index.ts
var rpc_exports = {};
_chunk2NC7LVYZjs.__export.call(void 0, rpc_exports, {
  RpcExitError: () => RpcExitError,
  RpcGMCallTypes: () => _chunk2NC7LVYZjs.RpcGMCallTypes,
  createRpcWorker: () => createRpcWorker,
  exposeRpc: () => _chunk2NC7LVYZjs.exposeRpc,
  getRpcWorkerData: () => getRpcWorkerData,
  wrapRpc: () => wrapRpc
});

// packages/native-federation-typescript/src/rpc/rpc-error.ts
var RpcExitError = class extends Error {
  constructor(message, code, signal) {
    super(message);
    this.code = code;
    this.signal = signal;
    this.name = "RpcExitError";
  }
};

// packages/native-federation-typescript/src/rpc/wrap-rpc.ts
function createControlledPromise() {
  let resolve = () => void 0;
  let reject = () => void 0;
  const promise = new Promise((aResolve, aReject) => {
    resolve = aResolve;
    reject = aReject;
  });
  return {
    promise,
    resolve,
    reject
  };
}
function wrapRpc(childProcess, options) {
  return async (...args) => {
    if (!childProcess.send) {
      throw new Error(`Process ${childProcess.pid} doesn't have IPC channels`);
    } else if (!childProcess.connected) {
      throw new Error(
        `Process ${childProcess.pid} doesn't have open IPC channels`
      );
    }
    const { id, once } = options;
    const {
      promise: resultPromise,
      resolve: resolveResult,
      reject: rejectResult
    } = createControlledPromise();
    const {
      promise: sendPromise,
      resolve: resolveSend,
      reject: rejectSend
    } = createControlledPromise();
    const handleMessage = (message) => {
      if ((message == null ? void 0 : message.id) === id) {
        if (message.type === "mf_resolve" /* RESOLVE */) {
          resolveResult(message.value);
          if (once) {
            removeHandlers();
          }
        } else if (message.type === "mf_reject" /* REJECT */) {
          rejectResult(message.error);
          if (once) {
            removeHandlers();
          }
        }
      }
    };
    const handleClose = (code, signal) => {
      rejectResult(
        new RpcExitError(
          code ? `Process ${childProcess.pid} exited with code ${code}${signal ? ` [${signal}]` : ""}` : `Process ${childProcess.pid} exited${signal ? ` [${signal}]` : ""}`,
          code,
          signal
        )
      );
      removeHandlers();
    };
    const removeHandlers = () => {
      childProcess.off("message", handleMessage);
      childProcess.off("close", handleClose);
    };
    childProcess.on("message", handleMessage);
    childProcess.on("close", handleClose);
    childProcess.on("error", (err) => {
      console.error(err);
    });
    childProcess.send(
      {
        type: "mf_call" /* CALL */,
        id,
        args
      },
      (error) => {
        if (error) {
          rejectSend(error);
          removeHandlers();
        } else {
          resolveSend(void 0);
        }
      }
    );
    return sendPromise.then(() => resultPromise);
  };
}

// packages/native-federation-typescript/src/rpc/rpc-worker.ts
var _child_process = require('child_process'); var child_process = _interopRequireWildcard(_child_process);
var _process = require('process'); var process = _interopRequireWildcard(_process);
var FEDERATION_WORKER_DATA_ENV_KEY = "VMOK_WORKER_DATA_ENV";
function uuid() {
  return new Array(4).fill(0).map(() => Math.floor(Math.random() * Number.MAX_SAFE_INTEGER).toString(16)).join("-");
}
function createRpcWorker(modulePath, data, memoryLimit, once) {
  const options = {
    env: {
      ...process.env,
      [FEDERATION_WORKER_DATA_ENV_KEY]: JSON.stringify(data || {})
    },
    stdio: ["inherit", "inherit", "inherit", "ipc"],
    serialization: "advanced"
  };
  if (memoryLimit) {
    options.execArgv = [`--max-old-space-size=${memoryLimit}`];
  }
  let childProcess, remoteMethod;
  const id = uuid();
  const worker = {
    connect(...args) {
      if (childProcess && !childProcess.connected) {
        childProcess.send({
          type: "mf_exit" /* EXIT */,
          id
        });
        childProcess = void 0;
        remoteMethod = void 0;
      }
      if (!(childProcess == null ? void 0 : childProcess.connected)) {
        childProcess = child_process.fork(modulePath, options);
        remoteMethod = wrapRpc(childProcess, { id, once });
      }
      if (!remoteMethod) {
        return Promise.reject("Worker is not connected - cannot perform RPC.");
      }
      return remoteMethod(...args);
    },
    terminate() {
      if (childProcess) {
        childProcess.send({
          type: "mf_exit" /* EXIT */,
          id
        });
        childProcess = void 0;
        remoteMethod = void 0;
      }
    },
    get connected() {
      return Boolean(childProcess == null ? void 0 : childProcess.connected);
    },
    get process() {
      return childProcess;
    },
    get id() {
      return id;
    }
  };
  return worker;
}
function getRpcWorkerData() {
  return JSON.parse(process.env[FEDERATION_WORKER_DATA_ENV_KEY] || "{}");
}

// packages/native-federation-typescript/src/lib/DtsWorker.ts
var DtsWorker = class {
  constructor(options) {
    this._options = _lodashclonedeepwith2.default.call(void 0, options, (_value, key) => {
      if (key === "manifest") {
        return false;
      }
    });
    this.removeUnSerializationOptions();
    this._rpcWorker = createRpcWorker(
      _path2.default.resolve(__dirname, "./lib/forkGenerateDts.js"),
      {},
      void 0,
      true
    );
    this._res = this._rpcWorker.connect(this._options);
    Promise.resolve(this._res).then(() => {
      this.exit();
    });
  }
  removeUnSerializationOptions() {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    if ((_b = (_a = this._options.remote) == null ? void 0 : _a.moduleFederationConfig) == null ? void 0 : _b.manifest) {
      (_d = (_c = this._options.remote) == null ? void 0 : _c.moduleFederationConfig) == null ? true : delete _d.manifest;
    }
    if ((_f = (_e = this._options.host) == null ? void 0 : _e.moduleFederationConfig) == null ? void 0 : _f.manifest) {
      (_h = (_g = this._options.host) == null ? void 0 : _g.moduleFederationConfig) == null ? true : delete _h.manifest;
    }
  }
  get controlledPromise() {
    return this._res;
  }
  exit() {
    var _a;
    (_a = this._rpcWorker) == null ? void 0 : _a.terminate();
  }
};

// packages/native-federation-typescript/src/lib/generateTypesInChildProcess.ts
async function generateTypesInChildProcess(options) {
  const dtsWorker = new DtsWorker(options);
  return dtsWorker.controlledPromise;
}

// packages/native-federation-typescript/src/lib/consumeTypes.ts
async function consumeTypes(options) {
  var _a;
  const DTSManagerConstructor = _chunk2NC7LVYZjs.getDTSManagerConstructor.call(void 0, 
    (_a = options.host) == null ? void 0 : _a.implementation
  );
  const dtsManager = new DTSManagerConstructor(options);
  await dtsManager.consumeTypes();
}





exports.rpc_exports = rpc_exports; exports.generateTypesInChildProcess = generateTypesInChildProcess; exports.consumeTypes = consumeTypes;
